package com.lz.modules.equipment.service;


import com.baomidou.mybatisplus.extension.service.IService;
import com.lz.common.utils.PageUtils;
import com.lz.modules.equipment.entity.OneCode;
import com.lz.modules.equipment.entity.OneCodeReq;

import java.util.List;

/**
* <p>
*  服务类
* </p>
*
* @author quyixiao
* @since 2020-07-29
*/
public interface OneCodeService extends IService<OneCode> {



	OneCode selectOneCodeById(Long id);


	Long insertOneCode(OneCode oneCode);


	int updateOneCodeById(OneCode oneCode);


	int updateCoverOneCodeById(OneCode oneCode);


	int deleteOneCodeById(Long id);


	Long insertOneCodes(List<String> codes);

	List<String> selectNoPrints();

	OneCode selectByCode(String code);

	Long updatePrintOneCodes(List<String> codes);

    PageUtils selectByReq(OneCodeReq req);
}